# TrimPy: A basic API implementation for Trimlight Select systems,
# Copyright (C) 2021-2022 Ethan Dye
#
# TrimPy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# TrimPy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TrimPy. If not, see <https://www.gnu.org/licenses/>.

from enum import Enum


class Trim(Enum):
    PORT = 8189
    START = 90
    END = 165
    CONN = 12
    MODE = 13
    SET_NAME = 14
    QUERY_PATTERN = 22
    CREATE_PATTERN = 6
    UPDATE_PATTERN = 5
    DELETE_PATTERN = 4
    PREVIEW_PATTERN = 19
    DISP = 3
    DOT_COUNT = 18



# TrimPy: A basic API implementation for Trimlight Select systems,
# Copyright (C) 2021-2022 Ethan Dye
#
# TrimPy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# TrimPy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TrimPy. If not, see <https://www.gnu.org/licenses/>.

from TrimPy import Trim
from random import randint


def randByte():
    while True:
        b = randint(0, 255)
        if ((b != Trim.START.value) and (b != Trim.END.value)):
            break
    return b


def validatePatternOptions(options, querySrc):
    if (((options.patName is None) or (options.animation is None) or
       (options.speed is None) or (options.brightness is None) or
       (options.count_one is None) or (options.count_two is None) or
       (options.count_three is None) or (options.count_four is None) or
       (options.count_five is None) or (options.count_six is None) or
       (options.count_seven is None) or (options.color_one is None) or
       (options.color_two is None) or (options.color_three is None) or
       (options.color_four is None) or (options.color_five is None) or
       (options.color_six is None) or (options.color_seven is None)) and
       (querySrc is None)):
        print('All pattern options must be provided, or a pattern number must be provided to copy values from.')
        return False
    return True



# TrimPy: A basic API implementation for Trimlight Select systems,
# Copyright (C) 2021-2022 Ethan Dye
#
# TrimPy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# TrimPy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TrimPy. If not, see <https://www.gnu.org/licenses/>.

from enum import Enum


class Animation(Enum):
    STATIC = 0
    CHASE_FORWARD = 1
    CHASE_BACKWARD = 2
    MIDDLE_TO_OUT = 3
    OUT_TO_MIDDLE = 4
    STROBE = 5
    FADE = 6
    COMET_FORWARD = 7
    COMET_BACKWARD = 8
    WAVE_FORWARD = 9
    WAVE_BACKWARD = 10
    SOLID_FADE = 11



# TrimPy: A basic API implementation for Trimlight Select systems,
# Copyright (C) 2021-2022 Ethan Dye
#
# TrimPy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# TrimPy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TrimPy. If not, see <https://www.gnu.org/licenses/>.

from .enum.pattern import Pattern
from .enum.trim import Trim
from .enum.mode import Mode
from .enum.animation import Animation
from .messages.message import formatConnMsg, formatModeMsg, formatDispMsg, formatNameMsg, formatQueryPatternMsg
from .messages.message import formatUpdatePatternMsg, formatPreviewPatternMsg, formatDeletePatternMsg, formatDotMsg
from .messages.message import checkPatternNumber
from .messages.parser import parseOptions

__all__ = ['Pattern', 'Trim', 'Mode', 'Animation', 'formatConnMsg', 'formatModeMsg', 'formatDispMsg', 'formatNameMsg',
           'formatQueryPatternMsg', 'formatUpdatePatternMsg', 'formatPreviewPatternMsg', 'formatDeletePatternMsg',
           'formatDotMsg', 'checkPatternNumber', 'parseOptions', '__version__']
__version__ = '1.0.0'



# TrimPy: A basic API implementation for Trimlight Select systems,
# Copyright (C) 2021-2022 Ethan Dye
#
# TrimPy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# TrimPy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TrimPy. If not, see <https://www.gnu.org/licenses/>.

from optparse import OptionParser, OptionGroup
import TrimPy
import socket
import sys


def main() -> int:
    parser = OptionParser(usage="Usage: TrimPy [options]")
    parser.add_option("-i", "--ip", help="IP address to connect to (required)")
    parser.add_option("-m", "--mode", help="set trimlight to mode: timer, or manual")
    parser.add_option("-p", "--pattern", help=f"set trimlight to pattern: {', '.join([p.name for p in TrimPy.Pattern])} "
                      "(also accepts custom values in the form of integers representing the pattern number)")
    parser.add_option("-n", "--set-name", dest="name", help="set trimlight device name (< 25 characters)")
    parser.add_option("-d", "--dot-count", dest="count", type="int", help="set trimlight device dot count to N (< 4096 dots)",
                      metavar="N")
    parser.add_option("-q", "--query-pattern", dest="query", type="int",
                      help="query trimlight for information about pattern number N", metavar="N")
    parser.add_option("-D", "--delete-pattern", dest="delete", type="int", help="delete pattern number N", metavar="N")
    parser.add_option("-v", "--verbose", action="store_true", default=False, help="make lots of noise [default: %default]")
    parser.add_option("-V", "--version", action="store_true", default=False, help="print version and exit")

    group = OptionGroup(parser, "Create/Update/Preview Pattern",
                        "create/update/preview a trimlight pattern to match your liking")
    group.add_option("--preview-pattern", dest="preview", action="store_true", default=False,
                     help="preview a pattern specified by the options below, "
                     "will not update pattern if using '--source-pattern'")
    group.add_option("--source-pattern", dest="src", type="int", help="source values from pattern number N", metavar="N")
    group.add_option("--dest-pattern", dest="dest", type="int", help="write updated values to pattern number N", metavar="N")
    group.add_option("--name", dest="patName", help="set pattern name (< 25 characters)")
    group.add_option("--animation", dest="animation", help="set animation style: "
                     f"{', '.join([a.name for a in TrimPy.Animation])}")
    group.add_option("--speed", dest="speed", type="int", help="set animation speed [0-255]")
    group.add_option("--brightness", dest="brightness", type="int", help="set brightness [0-255]")
    group.add_option("--color-one", nargs=3, type="int", help="set 'R G B' integer values for color index one",
                     metavar="R G B")
    group.add_option("--color-two", nargs=3, type="int", help="set 'R G B' integer values for color index two",
                     metavar="R G B")
    group.add_option("--color-three", nargs=3, type="int", help="set 'R G B' integer values for color index three",
                     metavar="R G B")
    group.add_option("--color-four", nargs=3, type="int", help="set 'R G B' integer values for for color index four",
                     metavar="R G B")
    group.add_option("--color-five", nargs=3, type="int", help="set 'R G B' integer values for color index five",
                     metavar="R G B")
    group.add_option("--color-six", nargs=3, type="int", help="set 'R G B' integer values for color index six",
                     metavar="R G B")
    group.add_option("--color-seven", nargs=3, type="int", help="set 'R G B' integer values for color index seven",
                     metavar="R G B")
    group.add_option("--count-one", type="int", help="set color index one to repeat N times [0-30]", metavar="N")
    group.add_option("--count-two", type="int", help="set color index two to repeat N times [0-30]", metavar="N")
    group.add_option("--count-three", type="int", help="set color index three to repeat N times [0-30]", metavar="N")
    group.add_option("--count-four", type="int", help="set color index four to repeat N times [0-30]", metavar="N")
    group.add_option("--count-five", type="int", help="set color index five to repeat N times [0-30]", metavar="N")
    group.add_option("--count-six", type="int", help="set color index six to repeat N times [0-30]", metavar="N")
    group.add_option("--count-seven", type="int", help="set color index seven to repeat N times [0-30]", metavar="N")
    parser.add_option_group(group)

    (options, args) = parser.parse_args()
    if (options.version is True):
        print("TrimPy", TrimPy.__version__)
        print('Copyright (C) 2021-2022 Ethan Dye')
        print('License GPLv3: GNU GPL version 3 <https://gnu.org/licenses/gpl.html>.\n')
        print('TrimPy is free software: you are free to change and redistribute it.')
        print('There is NO WARRANTY, to the extent permitted by law.')
        return 0
    elif (options.ip is None):
        parser.print_help()
        return 0

    trimSocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    # Connect to trimlight
    trimSocket.connect((options.ip, TrimPy.Trim.PORT.value))
    trimSocket.sendall(TrimPy.formatConnMsg(options.verbose))
    connData = trimSocket.recv(1024)

    nameLen = connData[3] + 4
    pixCntStart = nameLen + 2  # Skip padding after Device name
    pixCntEnd = nameLen + 4  # Pixel count is represented as two bytes in hex

    deviceName = connData[4:nameLen].decode("ASCII")
    pixelCount = int.from_bytes(connData[pixCntStart:pixCntEnd], 'big')

    if (options.verbose is True):
        print('Recieved:', connData.hex())
        print('Device Name:', deviceName)
        print('Pixel Count:', pixelCount)

    TrimPy.parseOptions(options, trimSocket)

    trimSocket.close()
    return 0


if __name__ == '__main__':
    sys.exit(main())



# TrimPy: A basic API implementation for Trimlight Select systems,
# Copyright (C) 2021-2022 Ethan Dye
#
# TrimPy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# TrimPy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TrimPy. If not, see <https://www.gnu.org/licenses/>.

from datetime import datetime
from TrimPy import Trim, Pattern, Mode
from .helpers import randByte, validatePatternOptions
from .parser import parsePatternOptions
from re import match


def formatConnMsg(verbose):
    pad1 = randByte()
    pad2 = randByte()
    pad3 = randByte()
    date = datetime.now()
    year = int(date.strftime("%y"))
    month = int(date.strftime("%m"))
    day = int(date.strftime("%d"))
    wkday = int(date.strftime("%w")) + 1  # Python zero indexes from Sunday for week day
    hour = int(date.strftime("%H"))
    minute = int(date.strftime("%M"))
    second = int(date.strftime("%S"))
    date = bytes([pad1, pad2, pad3, year, month, day, wkday, hour, minute, second])
    length = bytes([len(date) >> 8, len(date)])
    n = (pad3 << 5 | pad1 >> 3 & 31 & pad2).to_bytes(2, 'big')
    if (verbose is True):
        print('Verify byte:', n[1:2].hex())
    return bytes([Trim.START.value, Trim.CONN.value]) + length + date + bytes([Trim.END.value])


def formatModeMsg(m):
    mode = bytes([Mode.MANUAL.value]) if m == 'manual' else bytes([Mode.TIMER.value])
    length = bytes([len(mode) >> 8, len(mode)])
    return bytes([Trim.START.value, Trim.MODE.value]) + length + mode + bytes([Trim.END.value])


def formatDispMsg(p):
    if (p in Pattern.__members__):
        pattern = bytes([Pattern[p].value])
    else:
        pattern = bytes([int(p)])
    length = bytes([len(pattern) >> 8, len(pattern)])
    return bytes([Trim.START.value, Trim.DISP.value]) + length + pattern + bytes([Trim.END.value])


def formatNameMsg(n):
    name = bytearray(n, "ASCII")
    length = bytes([len(name) >> 8, len(name)])
    return bytes([Trim.START.value, Trim.SET_NAME.value]) + length + name + bytes([Trim.END.value])


def formatQueryPatternMsg(p):
    pattern = bytes([p])
    length = bytes([len(pattern) >> 8, len(pattern)])
    return bytes([Trim.START.value, Trim.QUERY_PATTERN.value]) + length + pattern + bytes([Trim.END.value])


def formatUpdatePatternMsg(trimSocket, options):
    querySrc = checkPatternNumber(trimSocket, options.src, options.verbose)
    queryDst = checkPatternNumber(trimSocket, options.dest, options.verbose)
    if (not validatePatternOptions(options, querySrc)):
        return

    p1 = bytearray([int(options.dest)])
    p2 = bytearray(options.patName.ljust(24, '\0'), "ASCII") if (options.patName is not None) else querySrc[2:26]
    p3 = querySrc[26:28] + parsePatternOptions(options, bytearray(querySrc[28:59]))
    request = p1 + p2 + p3
    length = bytes([len(request) >> 8, len(request)])
    cmd = Trim.CREATE_PATTERN.value if (queryDst is None) else Trim.UPDATE_PATTERN.value

    return bytes([Trim.START.value, cmd]) + length + request + bytes([Trim.END.value])


def formatPreviewPatternMsg(trimSocket, options):
    querySrc = checkPatternNumber(trimSocket, options.src, options.verbose)
    if (not validatePatternOptions(options, querySrc)):
        return

    request = (querySrc[26:28] + parsePatternOptions(options, bytearray(querySrc[28:59]))
               if (querySrc is not None) else parsePatternOptions(options, bytearray(32)))
    length = bytes([len(request) >> 8, len(request)])
    return bytes([Trim.START.value, Trim.PREVIEW_PATTERN.value]) + length + request + bytes([Trim.END.value])


def formatDeletePatternMsg(p):
    pattern = bytes([p])
    length = bytes([len(pattern) >> 8, len(pattern)])
    return bytes([Trim.START.value, Trim.DELETE_PATTERN.value]) + length + pattern + bytes([Trim.END.value])


def formatDotMsg(c):
    count = c.to_bytes(2, 'big')
    length = bytes([len(count) >> 8, len(count)])
    return bytes([Trim.START.value, Trim.DOT_COUNT.value]) + length + count + bytes([Trim.END.value])


def checkPatternNumber(trimSocket, num, verbose):
    if (num is not None):
        trimSocket.sendall(formatQueryPatternMsg(num))
        queryData = trimSocket.recv(1024)
        if verbose:
            print('Recieved:', queryData.hex())
        if (match(b'\x5a\xff.*\xff\xa5', queryData)):
            print('Pattern number provided does not exist!')
            return
        return queryData



# TrimPy: A basic API implementation for Trimlight Select systems,
# Copyright (C) 2021-2022 Ethan Dye
#
# TrimPy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# TrimPy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TrimPy. If not, see <https://www.gnu.org/licenses/>.

from enum import Enum


class Mode(Enum):
    TIMER = 0
    MANUAL = 1



# TrimPy: A basic API implementation for Trimlight Select systems,
# Copyright (C) 2021-2022 Ethan Dye
#
# TrimPy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# TrimPy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TrimPy. If not, see <https://www.gnu.org/licenses/>.

import TrimPy


def parseOptions(options, trimSocket):  # noqa: C901
    if (options.mode is not None):
        trimSocket.sendall(TrimPy.formatModeMsg(options.mode))
    if (options.pattern is not None):
        trimSocket.sendall(TrimPy.formatDispMsg(options.pattern.upper()))
    if (options.name is not None):
        trimSocket.sendall(TrimPy.formatNameMsg(options.name))
    if (options.query is not None):
        queryData = TrimPy.checkPatternNumber(trimSocket, options.query, options.verbose)
        parseQueryData(queryData)
    if (options.preview is not False):
        message = TrimPy.formatPreviewPatternMsg(trimSocket, options)
        if (message is not None):
            trimSocket.sendall(TrimPy.formatModeMsg('manual'))
            trimSocket.sendall(message)
    elif (options.dest is not None):
        message = TrimPy.formatUpdatePatternMsg(trimSocket, options)
        if (message is not None):
            trimSocket.sendall(message)
    if (options.delete is not None):
        trimSocket.sendall(TrimPy.formatDeletePatternMsg(options.delete))
    if (options.count is not None):
        trimSocket.sendall(TrimPy.formatDotMsg(options.count))


def parsePatternOptions(options, request):
    request[0:1] = bytes([TrimPy.Animation[options.animation].value]) if (options.animation is not None) else request[0:1]
    request[1:2] = bytes([options.speed]) if (options.speed is not None) else request[1:2]
    request[2:3] = bytes([options.brightness]) if (options.brightness is not None) else request[2:3]
    request[3:4] = bytes([options.count_one]) if (options.count_one is not None) else request[3:4]
    request[4:5] = bytes([options.count_two]) if (options.count_two is not None) else request[4:5]
    request[5:6] = bytes([options.count_three]) if (options.count_three is not None) else request[5:6]
    request[6:7] = bytes([options.count_four]) if (options.count_four is not None) else request[6:7]
    request[7:8] = bytes([options.count_five]) if (options.count_five is not None) else request[7:8]
    request[8:9] = bytes([options.count_six]) if (options.count_six is not None) else request[8:9]
    request[9:10] = bytes([options.count_seven]) if (options.count_seven is not None) else request[9:10]
    request[10:13] = bytes(options.color_one) if (options.color_one is not None) else request[10:13]
    request[13:16] = bytes(options.color_two) if (options.color_two is not None) else request[13:16]
    request[16:19] = bytes(options.color_three) if (options.color_three is not None) else request[16:19]
    request[19:22] = bytes(options.color_four) if (options.color_four is not None) else request[19:22]
    request[22:25] = bytes(options.color_five) if (options.color_five is not None) else request[22:25]
    request[25:28] = bytes(options.color_six) if (options.color_six is not None) else request[25:28]
    request[28:31] = bytes(options.color_seven) if (options.color_seven is not None) else request[28:31]

    return request


def parseQueryData(queryData):
    print('Pattern Name:', queryData[2:26].decode("ASCII"))
    print('Animation:', TrimPy.Animation(queryData[28]))
    print('Speed:', int(queryData[29]))
    print('Brightness:', int(queryData[30]))
    print('Dot Repetition:', int(queryData[31]), '|', int(queryData[32]), '|', int(queryData[33]), '|',
          int(queryData[34]), '|', int(queryData[35]), '|', int(queryData[36]), '|', int(queryData[37]))
    print('Dot RGB hex:', queryData[38:41].hex(), '|', queryData[41:44].hex(), '|', queryData[44:47].hex(), '|',
          queryData[47:50].hex(), '|', queryData[50:53].hex(), '|', queryData[53:56].hex(), '|',
          queryData[56:59].hex())



# TrimPy: A basic API implementation for Trimlight Select systems,
# Copyright (C) 2021-2022 Ethan Dye
#
# TrimPy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# TrimPy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TrimPy. If not, see <https://www.gnu.org/licenses/>.

from enum import Enum


# You may need to do some research / experimentation to figure out what your
# patterns correspond to. Custom patterns can also be accessed using this same
# format, they just come after the builtin patterns.
class Pattern(Enum):
    NEW_YEAR = 1
    ST_VALENTINE = 2
    ST_PATRICK = 3
    MOTHERS_DAY = 4
    INDEPENDENCE_DAY = 5
    DEFAULT = 6
    HALLOWEEN = 7
    THANKSGIVING = 8
    CHRISTMAS = 9



import setuptools
import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), 'src/'))
from TrimPy import __version__  # noqa: E402

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="TrimPy",
    version=__version__,
    author="Ethan Dye",
    author_email="mrtops03@gmail.com",
    description="A basic API implementation for Trimlight Select systems.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ecdye/TrimPy",
    project_urls={
        "Bug Tracker": "https://github.com/ecdye/TrimPy/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta"
    ],
    package_dir={"": "src"},
    packages=['TrimPy', 'TrimPy.enum', 'TrimPy.messages'],
    python_requires=">=3.6",
)
